package de.ozgcloud.eingang.semantik.enginebased.afm.intelliform;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.collections.MapUtils;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.semantik.common.ServiceKontoFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Component
@RequiredArgsConstructor
@Log4j2
public class IntelliFormRepresentationAdapter {

	static final String INTELLIFORM_TYPENAME = "http://xmlns.cit.de/intelliform/transaction";

	static final String FILE = "file";

	static final String HEADER_FIELD = "header";

	public static final List<String> HEADER_ATTRIBUTE_NAMES = List.of(
			"t:id",
			"t:timestamp",
			"t:form-id",
			"t:form",
			"t:sender",
			"t:customer",
			"t:customer-id",
			"t:client",
			"t:client-id"
	);
	public static final List<String> CUSTOM_HEADER_ATTRIBUTE_NAMES = List.of(
			"u:saml_legacypostkorbhandle",
			"u:saml_givenname",
			"u:saml_surname",
			"u:saml_placeofbirth",
			"u:saml_birthname",
			"u:saml_mail",
			"u:saml_telephonenumber",
			"u:saml_postaladdress",
			"u:saml_postalcode",
			"u:saml_localityname",
			ServiceKontoFactory.KEY_BAYERN_ID_TRUST_LEVEL
	);

	private static final Collector<Map.Entry<String, ?>, ?, Map<String, Object>> ORDERED_MAP_ENTRY_COLLECTOR = Collectors.toMap(
			Map.Entry::getKey,
			Map.Entry::getValue,
			(u, v) -> v,
			LinkedHashMap::new
	);

	private final XmlToJavaMapsMapper xmlToJavaMapsMapper;

	public boolean isResponsible(FormData formData) {
		return findIntelliFormXMLRepresentation(formData)
				.isPresent();
	}

	Optional<Document> findIntelliFormXMLRepresentation(FormData formData) {
		return streamXMLDocumentsFromRepresentations(formData)
				.filter(document -> {
					var element = document.getDocumentElement();
					var attribute = element.getAttribute("xmlns:t");
					return attribute.equals(INTELLIFORM_TYPENAME);
				})
				.findFirst();
	}

	Stream<Document> streamXMLDocumentsFromRepresentations(FormData formData) {
		return formData.getRepresentations().stream()
				.filter(representation -> representation.getContentType().contains("xml"))
				.flatMap(xmlRepresentation -> {
					try {
						return Stream.of(xmlToJavaMapsMapper.parseAsW3cDocument(xmlRepresentation.getContentStream()));
					} catch (TechnicalException exception) {
						return Stream.empty();
					}
				});
	}

	public FormData adaptByRepresentations(FormData formData) {
		var document = findIntelliFormXMLRepresentation(formData)
				.orElseThrow(() -> new TechnicalException("Expect to find IntelliForm XML representation!"));

		return FormData.builder()
				.representations(formData.getRepresentations())
				.attachments(formData.getAttachments())
				.formData(Stream.concat(
						getFormDataEntriesFromDocument(document, formData.getAttachments()),
						Map.of(
								HEADER_FIELD, createHeaderMap(document)
						).entrySet().stream()
				).collect(ORDERED_MAP_ENTRY_COLLECTOR))
				.build();
	}

	Stream<Map.Entry<String, Object>> getFormDataEntriesFromDocument(Document document, List<IncomingFileGroup> attachments) {
		Set<String> attachmentVendorIds = attachments.stream()
				.flatMap(group -> group.getFiles().stream())
				.map(IncomingFile::getVendorId)
				.collect(Collectors.toSet());

		return xmlToJavaMapsMapper.mapXmlToJavaMaps(document).entrySet()
				.stream()
				.filter(entry -> !isUploadElementWhichRefersToAttachment(entry.getValue(), attachmentVendorIds));
	}

	private boolean isUploadElementWhichRefersToAttachment(Object element, Set<String> attachmentVendorIds) {
		return findVendorIdInUploadElementCandidate(element)
				.map(attachmentVendorIds::contains)
				.orElse(false);
	}

	@SuppressWarnings("rawtypes")
	private Optional<String> findVendorIdInUploadElementCandidate(Object element) {
		if (element instanceof Map uploadElement) {
			return Optional.ofNullable(MapUtils.getMap(uploadElement, FILE))
					.flatMap(fileElement -> Optional.ofNullable(MapUtils.getString(fileElement, "id")));
		} else {
			return Optional.empty();
		}
	}

	Map<String, Object> createHeaderMap(Document document) {
		var element = document.getDocumentElement();
		return Stream.concat(HEADER_ATTRIBUTE_NAMES.stream(), CUSTOM_HEADER_ATTRIBUTE_NAMES.stream().filter(element::hasAttribute))
				.collect(Collectors.toMap(name -> name, element::getAttribute));
	}

}
