package de.ozgcloud.eingang.semantik.formbased.mantelantrag;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.UnaryOperator;
import java.util.stream.IntStream;

import jakarta.annotation.PostConstruct;

import org.apache.commons.collections.MapUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.boot.context.properties.bind.Bindable;
import org.springframework.boot.context.properties.bind.Binder;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import de.ozgcloud.eingang.semantik.formbased.FormBasedMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Component
@RequiredArgsConstructor
@Log4j2
public class MantelantragZustaendigeStelleMapper implements FormBasedMapper {
	static final String MANTELANTRAG_FORM_ID = "maa_mantelantrag/maa_mantelantrag";
	static final String ZUSTELLUNG_NACHRICHTENBROKER_FIELD = "zustellung_nachrichtenbroker";
	static final String ORGANISATIONSEINHEIT_ID_FIELD = "kontaktsystem_oeid";
	static final String BEZEICHNUNG_FIELD = "OrganisationseinheitenBEZEICHNUNG";
	static final String EMAIL_FIELD = "zust_emailadresse";
	static final String HAUSANSCHRIFT_STRASSE_FIELD = "zust_strasse";
	static final String HAUSANSCHRIFT_PLZ_FIELD = "zust_postleitzahl";
	static final String HAUSANSCHRIFT_ORT_FIELD = "zust_ort";
	static final String TELEFON_FIELD = "telefon";

	private final Environment environment;
	private Set<String> xtaIdentifiers;

	@PostConstruct
	void init() {
		xtaIdentifiers = new HashSet<>(getIdentifierList());
	}

	List<String> getIdentifierList() {
		return Binder.get(environment)
				.bind("ozgcloud.xta.identifiers", Bindable.listOf(String.class))
				.orElseGet(Collections::emptyList)
				.stream()
				.map(String::toLowerCase)
				.toList();
	}

	@Override
	public boolean isResponsible(FormData formData) {
		return Optional.ofNullable(formData.getHeader())
				.map(FormHeader::getFormId)
				.map(v -> v.startsWith(MANTELANTRAG_FORM_ID))
				.orElse(false);
	}

	@Override
	public FormData parseFormData(FormData formData) {
		return formData.toBuilder()
				.clearZustaendigeStelles()
				.zustaendigeStelles(getZustaendigeStelles(formData))
				.build();
	}

	List<ZustaendigeStelle> getZustaendigeStelles(FormData formData) {
		var fieldMap = formData.getFormData();
		var matchingSlots = getMatchingSlots(fieldMap);
		if (matchingSlots.isEmpty()) {
			logErrorForNoMatchingIdentifiers();
		}
		return matchingSlots.stream()
				.map(slotIndex -> getZustaendigeStelle(fieldMap, slotIndex))
				.toList();
	}

	ZustaendigeStelle getZustaendigeStelle(Map<String, Object> fieldMap, int slotIndex) {
		UnaryOperator<String> getValueOrEmpty = field -> getFieldByKeyOrEmpty(
				fieldMap,
				getNameForSlotIndex(field, slotIndex)
		);
		var organisationseinheitId = getValueOrEmpty.apply(ORGANISATIONSEINHEIT_ID_FIELD);
		if (organisationseinheitId.isBlank()) {
			logWarningForEmptyOrganisationseinheitId(slotIndex);
		}
		return ZustaendigeStelle.builder()
				.organisationseinheitenId(organisationseinheitId)
				.bezeichnung(getValueOrEmpty.apply(BEZEICHNUNG_FIELD))
				.email(getValueOrEmpty.apply(EMAIL_FIELD))
				.hausanschriftStrasse(getValueOrEmpty.apply(HAUSANSCHRIFT_STRASSE_FIELD))
				.hausanschriftPlz(getValueOrEmpty.apply(HAUSANSCHRIFT_PLZ_FIELD))
				.hausanschriftOrt(getValueOrEmpty.apply(HAUSANSCHRIFT_ORT_FIELD))
				.telefon(getValueOrEmpty.apply(TELEFON_FIELD))
				.build();
	}

	void logWarningForEmptyOrganisationseinheitId(int slotIndex) {
		LOG.warn("[Mantelantrag] OrganisationseinheitId for slot {} is empty!", slotIndex);
	}

	void logErrorForNoMatchingIdentifiers() {
		LOG.error("[Mantelantrag] No matching Zustaendigestelle found! For identifiers: '{}'.", Strings.join(xtaIdentifiers, ','));
	}

	List<Integer> getMatchingSlots(Map<String, Object> fieldMap) {
		return IntStream.range(0, 3)
				.filter(slotIndex -> xtaIdentifiers.contains(getXtaIdentifierOfSlot(fieldMap, slotIndex)))
				.boxed()
				.toList();
	}

	private String getXtaIdentifierOfSlot(Map<String, Object> fieldMap, int slotIndex) {
		return getFieldByKeyOrEmpty(fieldMap, getNameForSlotIndex(ZUSTELLUNG_NACHRICHTENBROKER_FIELD, slotIndex)).toLowerCase();
	}

	String getNameForSlotIndex(String name, int slotIndex) {
		return slotIndex > 0 ? "%s%d".formatted(name, slotIndex + 1) : name;
	}

	private String getFieldByKeyOrEmpty(Map<String, Object> fieldMap, String key) {
		return Optional.ofNullable(MapUtils.getString(fieldMap, key))
				.map(String::trim)
				.orElse("");
	}

}
