package de.ozgcloud.eingang.semantik.formbased.mantelantrag;

import static de.ozgcloud.eingang.common.formdata.FormDataTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.DynamicPropertyRegistry;
import org.springframework.test.context.DynamicPropertySource;

import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.Application;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.router.VorgangRemoteService;
import de.ozgcloud.eingang.semantik.SemantikAdapter;
import de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapterITCase;
import lombok.SneakyThrows;

@ActiveProfiles({ "local", "itcase" })
@SpringBootTest(classes = Application.class)
public class MantelantragITCase {

	private static final String FILE_NAME_XDOMEA = "mantelantrag/4620-EH6C_b3c9168a-6ae9-4361-8b2f-6837bb341021_Geschaeftsgang.Geschaeftsgang.0201.xml";
	private static final String FILE_NAME_MANTELANTRAG = "mantelantrag/4620-EH6C_7d703670-15b6-42b2-8cd1-88a7e4c494b9_Antrag_Max_Mustermann_SGBXII.xml";

	@MockBean
	VorgangRemoteService vorgangRemoteService;

	@SpyBean
	SemantikAdapter semantikAdapter;

	@Captor
	ArgumentCaptor<FormData> formDataCaptor;
	private FormData formData;

	@Captor
	ArgumentCaptor<Optional<String>> oeIdCaptor;

	@Autowired
	MantelantragZustaendigeStelleMapper mantelantragZustaendigeStelleMapper;

	static List<String> xtaIdentifiers = Collections.emptyList();

	@BeforeEach
	void mock() {
		clearInvocations(vorgangRemoteService);
		doReturn(VORGANG_ID).when(vorgangRemoteService).createVorgang(any(FormData.class), any());
		formData = prepareTestData();
		mantelantragZustaendigeStelleMapper.init();
	}

	@DynamicPropertySource
	static void dynamicProperties(DynamicPropertyRegistry registry) {
		registry.add("ozgcloud.xta.identifiers", () -> xtaIdentifiers);
	}

	@DisplayName("without matching zustaendige stelle")
	@Nested
	class TestWithoutMatchingZustaendigeStelle {

		@BeforeAll
		static void setup() {
			xtaIdentifiers = List.of("afmsh:unknown");
		}

		@DisplayName("should create one vorgang")
		@Test
		void shouldCreateOneVorgang() {
			semantikAdapter.processFormData(formData);

			verify(vorgangRemoteService, times(1)).createVorgang(formDataCaptor.capture(), oeIdCaptor.capture());
			assertThat(oeIdCaptor.getValue()).isEmpty();
		}
	}

	@DisplayName("with one matching zustaendige stelle")
	@Nested
	class TestWithOneMatchingZustaendigeStelle {
		@BeforeAll
		static void setup() {
			xtaIdentifiers = List.of("afmsh:ozg-cloud-utopia-test");
		}

		@DisplayName("should create one vorgang")
		@Test
		void shouldCreateOneVorgang() {
			semantikAdapter.processFormData(formData);

			verify(vorgangRemoteService, times(1)).createVorgang(formDataCaptor.capture(), oeIdCaptor.capture());
			assertThat(oeIdCaptor.getValue()).contains("123");
		}
	}

	@DisplayName("with two matching zustaendige stelle")
	@Nested
	class TestWithTwoMatchingZustaendigeStelle {

		@BeforeAll
		static void setup() {
			xtaIdentifiers = List.of("afmsh:ozg-cloud-utopia-test", "gae:ozg-cloud-itcase-test");
		}

		@DisplayName("should create two vorangs with partially matching zustaendigeStelle")
		@Test
		void shouldCreateTwoVorangsWithPartiallyMatchingZustaendigeStelle() {
			semantikAdapter.processFormData(formData);

			verify(vorgangRemoteService, times(2)).createVorgang(formDataCaptor.capture(), oeIdCaptor.capture());
			assertThat(oeIdCaptor.getAllValues()).contains(Optional.of("123"), Optional.of("444"));
		}
	}

	@DisplayName("with three matching zustaendige stelles")
	@Nested
	class TestWithThreeMatchingZustaendigeStelles {

		@BeforeAll
		static void setup() {
			xtaIdentifiers = List.of("afmsh:ozg-cloud-utopia-test", "gae:ozg-cloud-itcase-test", "afmsh:ozg-cloud-bad-segeberg-kreis");
		}

		@DisplayName("should create three vorgangs")
		@Test
		void shouldCreateThreeVorgangs() {
			semantikAdapter.processFormData(formData);

			verify(vorgangRemoteService, times(3)).createVorgang(formDataCaptor.capture(), oeIdCaptor.capture());
			assertThat(oeIdCaptor.getAllValues()).contains(Optional.of("123"), Optional.of("321"), Optional.of("444"));
		}
	}

	@DisplayName("should return first vorgangId")
	@Test
	void shouldReturnFirstVorgangId() {
		var result = semantikAdapter.processFormData(formData);

		assertThat(result).isEqualTo(VORGANG_ID);
	}

	private FormData prepareTestData() {
		var xdomeaFile = IncomingFileTestFactory.createBuilder()
				.name(FILE_NAME_XDOMEA)
				.file(asFile(FILE_NAME_XDOMEA))
				.size(getFileSize(FILE_NAME_XDOMEA))
				.contentType("application/xml")
				.build();
		var mantelantragFile = IncomingFileTestFactory.createBuilder()
				.name(FILE_NAME_MANTELANTRAG)
				.file(asFile(FILE_NAME_MANTELANTRAG))
				.size(getFileSize(FILE_NAME_MANTELANTRAG))
				.contentType("application/xml")
				.build();

		return FormData.builder().representations(List.of(xdomeaFile, mantelantragFile)).build();
	}

	@SneakyThrows
	private long getFileSize(String fileName) {
		Path filePath = Path.of(FormSolutionsEngineBasedAdapterITCase.class.getClassLoader().getResource(fileName).toURI());
		return Files.size(filePath);
	}

	private File asFile(String path) {
		return TempFileUtils.writeTmpFile(TestUtils.loadFile(path));
	}
}
